/*
 * Project name:
     tRF click (Using mikroE's tRF click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140416:
       - initial release (FJ);
 * Description:
      A simple example of using tRF click board, which will send the text string
      via the Telit's LE70-868 RF module when PORTD.B3 button is pressed and
      display the received string on the Lcd.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     Ext. Modules:    tRF click board - ac:tRF_click
                      2x16 Lcd character display - ac:Lcd
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Place tRF click board in the mikroBUS socket 1 on the EasyAVR v7 board.
     - Pull-down PORTD.B3, Pull-up SW1.4 on the EasyAVR v7 board.
     - Put power supply jumper (J5) on the EasyAVR v7 board in the 3.3V position.
 */


// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

sbit Send_Button at PIND3_bit;
sbit Send_Button_Direction at DDD3_bit;

char rxBuff[16];                         // Buffer string for storing data
char rxCnt = 0;                          // Counter for data writen in buffer string
unsigned short Received = 0;             // Received data flag

// Interrupt routine
void interrupt() iv IVT_ADDR_USART__RXC ics ICS_AUTO {
  char tmp;                              // Temporary variable declaration
  if(RXC_bit == 1) {                     // Check for Receive Interrupt Flag bit
    tmp = UART_Read();                   // Store received data in the temporary variable
    if ((tmp == '\r') || (rxCnt > 15)) { // Check if data counter is larger than the buffer string size
                                         // or the received data is equal to the Carriage Return character
      rxBuff[rxCnt] = 0;                 // Terminate buffer string
      rxCnt = 0;                         // Reset data counter
      Received = 1;                      // Set Received data flag
    }
    else{
      rxBuff[rxCnt] = tmp;               // Populate buffer string with received data
      rxCnt = rxCnt + 1;                 // Increase data counter
    }
  }
}

// MCU initialization
void MCU_Init(){
  Send_Button_Direction = 0;             // Set PORTD.B3 pin as input
  
  UART1_Init(19200);                     // Initialize UART module at 19200 bps
  Delay_ms(100);                         // Wait for UART module to stabilize

  Lcd_Init();                            // Initialize LCD display
  Lcd_Cmd(_LCD_CURSOR_OFF);              // Cursor off
  Lcd_Cmd(_LCD_CLEAR);                   // Clear display

  RXCIE_bit = 1;                         // Turn ON interrupt on UART1 receive
  SREG_I_bit = 1;                        // Enable Interrupts
}

void main() {
  MCU_Init();                            // Initialize the MCU
  
  Lcd_Out(1, 1, "tRFclick example");     // Display starting message on the Lcd
  
  while(1) {
    if(Button(&PIND, 3, 1, 1))  {        // If PORTD.D3 is pressed :
      UART1_Write_Text("EchoTest\r");    // send text string
      Delay_ms(250);                     // wait for 250ms
    }
    
    if(Received){                        // If data is received via UART :
      Lcd_cmd(_LCD_SECOND_ROW);          // set cursor in the second row
      Lcd_Out(2, 1, "                "); // clear second row
      Lcd_Out(2, 1, rxBuff);             // display the received data in the second row
      Received = 0;                      // clear received data flag
    }
  }
}